@@ -7,6 +7,7 @@
 
 #include "eval_nvmix_integral.h"
 #include "eval_densmix_integrand.h"
+#include "eval_gdensmix_integrand.h"
 #include "logsumexp2.h"
 #include "precond.h"
 
@@ -14,6 +15,8 @@
 static const R_CallMethodDef callMethods[] = {
     {"eval_nvmix_integral", (DL_FUNC) &eval_nvmix_integral, 15},
     {"eval_densmix_integrand", (DL_FUNC) &eval_densmix_integrand, 7},
+    {"eval_gdensmix_integrand_returnall", (DL_FUNC) &eval_gdensmix_integrand_returnall, 8},
+        {"eval_gdensmix_integrand_LSE", (DL_FUNC) &eval_gdensmix_integrand_LSE, 8},
     {"logsumexp2", (DL_FUNC) &logsumexp2, 3},
     {NULL, NULL, 0}
 };
